!----------------------------------------------------------------------
 
! Code converted using TO_F90 by Alan Miller
! Date: 2000-07-11  Time: 12:08:48

! 6. EX2.FOR, EXAMPLE 2

! The data for this example were obtained as digitized observations on the
! outline of the nerve-head in a subject's eye.  There are 32 data (NDAT),
! one functional relation (NEQ), two variables (NVAR), and five parameters
! (NPAR).  The functional relation describes an ellipse with the following
! equation

!  F = T3*(XI1-T1)**2 + 2*T4*(XI1-T1)*(XI2-T2) + T5*(XI2-T2)**2 - 1 = 0

! where F is the functional value which numerically must be brought to zero,
! T1,T2,T3,T4 and T5 are the functional parameters the values of which are
! to be estimated.  XI1 and XI2 are the estimated true values of the measured
! variables.  The T's are represented by the Greek letter theta in the paper
! and the XI's by xi.  The parameters which describe the magnitude of the
! problem are NDAT = 32, NPAR = 4, NVAR = 2.  The program parameters were
! set at CRIT1 = 1.E-6 and CRIT2 = 1.E-5.  NITER1 was 50 and NITER2 15.  The
! full Fortran code as used is listed under EX2.FOR.  The data in the format
! used by the program is under EX2.DAT.  The output is listed under EX2.OUT.

! EX2.FOR


! FITTING OF ELLIPSE BY EVMS  91/3/25 PMR

PROGRAM ex2
USE Errors_in_Variables
IMPLICIT NONE

INTEGER    :: i, ifault(2), j
REAL (dp)  :: bs(1,2), DATA(32,2), est(32,2), g1(5,5), g3(5,5),   &
              resid(32,2), theta(5)
REAL (dp)  :: v(2,2) = RESHAPE( (/ 1.0_dp, 0.0_dp, 0.0_dp, 1.0_dp /), (/ 2,2 /))

OPEN(21, FILE='ex2.dat', STATUS='OLD')
OPEN(31, FILE='ex2.res')

READ (21,*) ((DATA(i,j),j=1,2),i=1,32)
DO  i=1,32
  DO  j=1,2
    DATA(i,j)=.001*DATA(i,j)
  END DO
END DO

READ (21,*) theta
CALL evms (bs, DATA, est, g1, g3, ifault, 32, 5, 2, resid, theta, v)
DO  i=1,32
  DO  j=1,2
    DATA (i,j)=1000.*DATA(i,j)
    est(i,j)=1000.*est(i,j)
    resid(i,j)=1000.*resid(i,j)
  END DO
END DO

WRITE (31,50)
50 FORMAT (//'FAULT STATUS')
WRITE (31,60) ifault
60 FORMAT (2I5)
WRITE (31,70)
70 FORMAT (//'PARAMETER ESTIMATES')
WRITE (31,80) theta
80 FORMAT (5F9.4)
WRITE (31,90)
90 FORMAT (//'INFORMATION MATRIX')
WRITE (31,80) g1
WRITE (31,100)
100 FORMAT (//'INVERSE OF INFORMATION MATRIX')
WRITE (31,80) g3
WRITE (31,110)
110 FORMAT (//t7, 'DATA', t22, 'ESTIMATES', t45, 'RESIDUALS')
WRITE (31,120) ((DATA(i,j),j=1,2),(est(i,j),j=1,2),(resid(i,j),j= 1,2),i=1,32)
120 FORMAT (2F7.0, 2F11.4, ' ', 2F10.4)
STOP
END PROGRAM ex2



SUBROUTINE bf (b, f, theta, xi)
IMPLICIT NONE
INTEGER, PARAMETER  :: dp = SELECTED_REAL_KIND(12, 60)

REAL (dp), INTENT(OUT)  :: b(:,:)
REAL (dp), INTENT(OUT)  :: f(:)
REAL (dp), INTENT(IN)   :: theta(:)
REAL (dp), INTENT(IN)   :: xi(:)

REAL (dp)  :: w1, w2

! CALCULATE F
w1=xi(1) - theta(1)
w2=xi(2) - theta(2)
f(1)=theta(3)*w1*w1 + 2.d0*theta(4)*w1*w2 + theta(5)*w2*w2 - 1.d0

! CALCULATE B
b(1,1)=2.d0*(w1*theta(3) + w2*theta(4))
b(1,2)=2.d0*(w1*theta(4) + w2*theta(5))
RETURN
END SUBROUTINE bf



SUBROUTINE zed (b, f, theta, xi, z)
IMPLICIT NONE
INTEGER, PARAMETER  :: dp = SELECTED_REAL_KIND(12, 60)

REAL (dp), INTENT(IN)   :: b(:,:)
REAL (dp), INTENT(IN)   :: f(:)
REAL (dp), INTENT(IN)   :: theta(:)
REAL (dp), INTENT(IN)   :: xi(:)
REAL (dp), INTENT(OUT)  :: z(:,:)

REAL (dp)  :: w1, w2

w1=xi(1) - theta(1)
w2=xi(2) - theta(2)
z(1,1)=-2.d0*(w1*theta(3) + w2*theta(4))
z(1,2)=-2.d0*(w1*theta(4) + w2*theta(5))
z(1,3)=w1*w1
z(1,4)=2.d0*w1*w2
z(1,5)=w2*w2
RETURN
END SUBROUTINE zed
